"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const sift_1 = require("sift");
const log = require("./log");
const db = require("./db");
const _1 = require("./_");
const Event_1 = require("./Event");
const queue_1 = require("./queue");
const ProgramItem_1 = require("./ProgramItem");
class Program {
    constructor() {
        this._items = [];
        this._programGCInterval = _1.default.config.server.programGCInterval || 1000 * 60 * 15;
        this._load();
        setTimeout(this._gc.bind(this), this._programGCInterval);
    }
    static get(id) {
        return _1.default.program.get(id);
    }
    static exists(id) {
        return _1.default.program.exists(id);
    }
    static findByQuery(query) {
        return _1.default.program.findByQuery(query);
    }
    static findByServiceId(serviceId) {
        return _1.default.program.findByServiceId(serviceId);
    }
    static all() {
        return _1.default.program.items;
    }
    get items() {
        return this._items;
    }
    add(item, firstAdd = false) {
        if (this.get(item.id) !== null) {
            return;
        }
        const removedIds = [];
        if (firstAdd === false) {
            _1.default.program.findConflicts(item.data.networkId, item.data.serviceId, item.data.startAt, item.data.startAt + item.data.duration).forEach(conflictedItem => {
                this.remove(conflictedItem);
                log.debug("ProgramItem#%d (networkId=%d, eventId=%d) has removed for redefine to ProgramItem#%d (eventId=%d)", conflictedItem.data.id, conflictedItem.data.networkId, conflictedItem.data.eventId, item.data.id, item.data.eventId);
                removedIds.push(conflictedItem.data.id);
            });
        }
        this._items.push(item);
        if (firstAdd === false) {
            Event_1.default.emit("program", "create", item.data);
            removedIds.forEach(id => Event_1.default.emit("program", "redefine", { from: id, to: item.data.id }));
        }
        this.save();
    }
    get(id) {
        const l = this._items.length;
        for (let i = 0; i < l; i++) {
            if (this._items[i].id === id) {
                return this._items[i];
            }
        }
        return null;
    }
    remove(item) {
        const index = this._items.indexOf(item);
        if (index !== -1) {
            this._items.splice(index, 1);
            this.save();
        }
    }
    exists(id) {
        return this.get(id) !== null;
    }
    findByQuery(query) {
        return sift_1.default(query, this._items);
    }
    findByServiceId(serviceId) {
        const items = [];
        const l = this._items.length;
        for (let i = 0; i < l; i++) {
            if (this._items[i].data.serviceId === serviceId) {
                items.push(this._items[i]);
            }
        }
        return items;
    }
    findConflicts(networkId, serviceId, start, end) {
        const items = [];
        const l = this._items.length;
        for (let i = 0; i < l; i++) {
            const item = this._items[i];
            if (item.data.networkId === networkId &&
                item.data.serviceId === serviceId &&
                item.data.startAt >= start &&
                item.data.startAt < end) {
                items.push(item);
            }
        }
        return items;
    }
    findByNetworkIdAndReplace(networkId, programs) {
        return __awaiter(this, void 0, void 0, function* () {
            let count = 0;
            for (let i = this._items.length - 1; i >= 0; i--) {
                if (this._items[i].data.networkId === networkId) {
                    this._items.splice(i, 1);
                    --count;
                }
            }
            for (const program of programs) {
                this.add(new ProgramItem_1.default(program), true);
                ++count;
            }
            log.debug("programs replaced (networkId=%d, count=%d)", networkId, count);
            this.save();
        });
    }
    save() {
        clearTimeout(this._saveTimerId);
        this._saveTimerId = setTimeout(() => this._save(), 3000);
    }
    _load() {
        log.debug("loading programs...");
        const now = Date.now();
        let dropped = false;
        db.loadPrograms().forEach(program => {
            if (typeof program.networkId === "undefined") {
                dropped = true;
                return;
            }
            if (now > (program.startAt + program.duration)) {
                dropped = true;
                return;
            }
            this.add(new ProgramItem_1.default(program), true);
        });
        if (dropped) {
            this.save();
        }
    }
    _save() {
        log.debug("saving programs...");
        db.savePrograms(this._items.map(program => program.data));
    }
    _gc() {
        log.debug("Program GC has queued");
        queue_1.default.add(() => __awaiter(this, void 0, void 0, function* () {
            const now = Date.now();
            let count = 0;
            this._items.forEach(program => {
                if (now > (program.data.startAt + program.data.duration)) {
                    ++count;
                    this.remove(program);
                }
            });
            setTimeout(this._gc.bind(this), this._programGCInterval);
            log.info("Program GC has finished and removed %d programs", count);
        }));
    }
}
exports.default = Program;

//# sourceMappingURL=Program.js.map
